
`resetall

//AHB arbiter demo
module ahb_arbiter_demo
(
  //UART0
  UART0RXD,
  UART0TXD,
  //SPI-Flash Memory
  FLASH_SPI_CSN,
  FLASH_SPI_MISO,
  FLASH_SPI_MOSI,
  FLASH_SPI_CLK,
  //
  HCLK,		//System clock
  hwRstn	//System reset
);

input HCLK;          // System clock
input hwRstn;        // System reset

//UART0
input UART0RXD;
output UART0TXD;

//SPI-Flash
inout FLASH_SPI_CSN;
inout FLASH_SPI_MISO;
inout FLASH_SPI_MOSI;
inout FLASH_SPI_CLK;


//MCU AHB Master [1]
//AHB Arbiter Master [1]
wire [31:0] AHB2HRDATA;      // Data from slave to master
wire        AHB2HREADYOUT;   // Slave ready signal
wire [1:0]  AHB2HRESP;       // Slave response signal  
wire [1:0]  AHB2HTRANS;      // Transfer type
wire [2:0]  AHB2HBURST;      // Burst type
wire [3:0]  AHB2HPROT;       // Transfer protection bits
wire [2:0]  AHB2HSIZE;       // Transfer size
wire        AHB2HWRITE;      // Transfer direction
wire        AHB2HMASTLOCK;   // Transfer is a locked transfer
wire        AHB2HREADYMUX;   // Ready mux signal
wire [3:0]  AHB2HMASTER;     // Transfer is a locked transfer
wire [31:0] AHB2HADDR;       // Transfer address
wire [31:0] AHB2HWDATA;      // Data from master to slave
wire        AHB2HSEL;        // Select
wire        AHB2HCLK;        // Bus clock signal
wire        AHB2HRESET;      // Bus reset signal

//AHB Arbiter Slave
//0x80000000
wire [31:0] SHRDATAM0 ;
wire        SHREADYOUTM0 ;
wire [1:0]  SHRESPM0 ;
wire        SHSELM0 ;
wire [31:0] SHADDRM0 ;
wire [1:0]  SHTRANSM0 ;
wire        SHWRITEM0 ;
wire [2:0]  SHSIZEM0 ;
wire [2:0]  SHBURSTM0 ;
wire [3:0]  SHPROTM0 ;
wire [3:0]  SHMASTERM0 ;
wire [31:0] SHWDATAM0 ;
wire        SHMASTLOCKM0 ;
wire        SHREADYMUXM0 ;


//Gowin_EMPU_M1 instantiation
Gowin_EMPU_M1_Top u_Gowin_EMPU_M1_Top
(
  .LOCKUP(),
  .UART0RXD(UART0RXD),
  .UART0TXD(UART0TXD),
  .AHB1HRDATA(AHB2HRDATA),       // Data from slave to master
  .AHB1HREADYOUT(AHB2HREADYOUT), // Slave ready signal, from slave, 1'b1
  .AHB1HRESP(AHB2HRESP),         // Slave response signal  
  .AHB1HTRANS(AHB2HTRANS),       // Transfer type
  .AHB1HBURST(AHB2HBURST),       // Burst type
  .AHB1HPROT(AHB2HPROT),         // Transfer protection bits
  .AHB1HSIZE(AHB2HSIZE),         // Transfer size
  .AHB1HWRITE(AHB2HWRITE),       // Transfer direction
  .AHB1HREADYMUX(AHB2HREADYMUX), // Ready mux
  .AHB1HMASTLOCK(AHB2HMASTLOCK), // Transfer is a locked transfer
  .AHB1HMASTER(AHB2HMASTER),     // Transfer is a locked transfer
  .AHB1HADDR(AHB2HADDR),         // Transfer address
  .AHB1HWDATA(AHB2HWDATA),       // Data from master to slave   AHB2HSEL,
  .AHB1HSEL(AHB2HSEL),           // Select
  .AHB1HCLK(AHB2HCLK),           // Clock
  .AHB1HRESET(AHB2HRESET),       // Reset
  .FLASH_SPI_HOLDN(),
  .FLASH_SPI_CSN(FLASH_SPI_CSN),
  .FLASH_SPI_MISO(FLASH_SPI_MISO),
  .FLASH_SPI_MOSI(FLASH_SPI_MOSI),
  .FLASH_SPI_WPN(),
  .FLASH_SPI_CLK(FLASH_SPI_CLK),
  .HCLK(HCLK),		             //System Clock
  .hwRstn(hwRstn)	             //System Reset
);


//Gowin_AHB_Arbiter instantiation
Gowin_AHB_Arbiter_Top u_Gowin_AHB_Arbiter_Top 
(
  .HCLK(AHB2HCLK),
  .HRESETn(AHB2HRESET),
  .MHSELS0(AHB2HSEL),
  .MHADDRS0(AHB2HADDR),
  .MHTRANSS0(AHB2HTRANS),
  .MHWRITES0(AHB2HWRITE),
  .MHSIZES0(AHB2HSIZE),
  .MHBURSTS0(AHB2HBURST),
  .MHPROTS0(AHB2HPROT),
  .MHMASTERS0(AHB2HMASTER),
  .MHWDATAS0(AHB2HWDATA),
  .MHMASTLOCKS0(AHB2HMASTLOCK),
  .MHREADYS0(AHB2HREADYMUX),
  .MHRDATAS0(AHB2HRDATA),
  .MHREADYOUTS0(AHB2HREADYOUT),
  .MHRESPS0(AHB2HRESP),
  .SHRDATAM0(SHRDATAM0),
  .SHREADYOUTM0(SHREADYOUTM0),
  .SHRESPM0(SHRESPM0),
  .SHSELM0(SHSELM0),
  .SHADDRM0(SHADDRM0),
  .SHTRANSM0(SHTRANSM0),
  .SHWRITEM0(SHWRITEM0),
  .SHSIZEM0(SHSIZEM0),
  .SHBURSTM0(SHBURSTM0),
  .SHPROTM0(SHPROTM0),
  .SHMASTERM0(SHMASTERM0),
  .SHWDATAM0(SHWDATAM0),
  .SHMASTLOCKM0(SHMASTLOCKM0),
  .SHREADYMUXM0(SHREADYMUXM0)
);

//Gowin_AHB_Multiplication instantiation
Gowin_AHB_Multiplication u_Gowin_AHB_Multiplication
(
	.AHB_HRDATA(SHRDATAM0),
	.AHB_HREADY(SHREADYOUTM0),  //Ready signal, slave to MCU master, 1'b1
	.AHB_HRESP(SHRESPM0),       //Respone signal, slave to MCU master
	.AHB_HTRANS(SHTRANSM0),
	.AHB_HBURST(SHBURSTM0),
	.AHB_HPROT(SHPROTM0),
	.AHB_HSIZE(SHSIZEM0),
	.AHB_HWRITE(SHWRITEM0),
	.AHB_HMASTLOCK(SHMASTLOCKM0),
	.AHB_HMASTER(SHMASTERM0),
	.AHB_HADDR(SHADDRM0[11:0]),
	.AHB_HWDATA(SHWDATAM0),
	.AHB_HSEL(SHSELM0),
	.AHB_HCLK(AHB2HCLK),
	.AHB_HRESETn(AHB2HRESET)
);

endmodule